package iMtest;

import oracle.ord.im.OrdImage;
import oracle.ord.im.OrdImageSignature;
// set CLASSPATH=$ORACLE_HOME/ord/jlib/ordim.jar;
// set CLASSPATH=$ORACLE_HOME/jdbc/lib/ojdbc14.jar;
// set CLASSPATH=$ORACLE_HOME/sqlj/lib/runtime12.jar;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import oracle.jdbc.OracleResultSet;
import oracle.jdbc.OraclePreparedStatement;
// nepotrebne :)
import java.sql.DatabaseMetaData;
import java.io.IOException;


// source2 =====================================================================
public class Source2 {
    // DB connection
    private Connection conn;
    private Statement stmt;
    
    // (de)constructor ---------------------------------------------------------
    public Source2() throws SQLException {
        // register the oracle jdbc driver with the JDBC driver manager 
        DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
        // connect pcuifs1.fit.vutbr.cz
        conn = DriverManager.getConnection("jdbc:oracle:thin:login/password@pcuifs1.fit.vutbr.cz:1521:stud" );

        // Note: it is CRITICAL to set the autocommit to false so that two-phase select-commit of BLOBs can occur.
        conn.setAutoCommit(false);
        
        // create a JDBC Statement object to execute SQL in the database
        stmt = conn.createStatement();
        
        // Test: Connection driver version
        // System.out.println("JDBC verze (z metadat): " + conn.getMetaData().getDriverVersion());
    }
    protected void finalize() throws SQLException {
/*
        // restore database to its original state - drop image table and sequence
        stmt.executeQuery("DROP TABLE image_table");
        stmt.executeQuery("DROP SEQUENCE image_sequence");
        // commit all our changes
        conn.commit();
*/
        // close the database connection to release all the resources
        conn.close();
        stmt.close();
    }
    
    // create a table with an key and ORDImage column to store images ----------
    public void tableCreate() throws SQLException {

        String SQLquery = "CREATE TABLE image_table (id number primary key, " + 
                   "image ordsys.ordimage, signature ordsys.ordimagesignature)";

        // execute the SQL
        stmt.execute(SQLquery);
        
        // create sequence
        SQLquery = "CREATE SEQUENCE image_sequence minvalue 1 maxvalue 999999999999999999999999999 increment by 1 start with 1 cache 20 noorder nocycle";
        stmt.execute(SQLquery);
        
        // commit all our changes
        conn.commit();
    }
    
    // upload an image from file into a table ----------------------------------   
    public void imageUpload(String fileName) throws SQLException, IOException {
        String SQLquery;
        OracleResultSet rset;

        // get image_sequence.nextval
        SQLquery = ("SELECT image_sequence.nextval FROM dual");
        rset = (OracleResultSet)stmt.executeQuery(SQLquery);
        rset.next();
        int nextval = rset.getInt("nextval");

        // insert a row (1, initialized ORDImage) into image_table
        SQLquery = "INSERT INTO image_table (id, image, signature) VALUES (" + nextval + 
             ", ordsys.ordimage.init(), ordsys.ordimagesignature.init())";
        stmt.execute(SQLquery);
        // System.out.println(SQLquery + " ... done.");        

        // select the new ORDImage into a java proxy OrdImage object (imageProxy)
        // Note: for update (since we will be uploading data into the ORDImages underlying BLOB)
        String rowSelectSQL = "SELECT image, signature FROM image_table WHERE id = " + nextval + " FOR UPDATE";
        rset = (OracleResultSet)stmt.executeQuery(rowSelectSQL);
        rset.next();
        OrdImage imageProxy = (OrdImage)rset.getCustomDatum("image", OrdImage.getFactory());
        // deprecated (?) use: OrdImage imageProxy = (OrdImage)rset.getORAData("image", OrdImage.getORADataFactory());
        OrdImageSignature signatureProxy = (OrdImageSignature)rset.getCustomDatum("signature", OrdImageSignature.getFactory());
        // deprecated
        rset.close();
        
        // load the image data from the goats.gif file into the ORDImage object 
        // Note: and by extension into the database :-)
        imageProxy.loadDataFromFile(fileName);

        // automatically detect the images height, width, file format, and so on
        // Note: the proxy object forwards the request to the database to execute ORDImage.setProperties() on the server
        imageProxy.setProperties();
        
        // generate an image signature for the specified image
        signatureProxy.generateSignature(imageProxy);

        // deprecated (?)
        String updateSQL = "UPDATE image_table SET image=?, signature=? where id=" + nextval;
        OraclePreparedStatement opstmt = (OraclePreparedStatement)conn.prepareStatement(updateSQL);
        opstmt.setCustomDatum(1, imageProxy);
        opstmt.setCustomDatum(2, signatureProxy);       
        opstmt.execute();
        opstmt.close();

        // commit all our changes
        conn.commit(); 
        // System.out.println("Commit ... done.");        
    }
    
    // create thumbnail and change format --------------------------------------
    public void imageConversion() throws SQLException {
        String SQLquery;
        OracleResultSet rset;
        
        // get image_sequence.nextval
        SQLquery = ("SELECT image_sequence.nextval FROM dual");
        rset = (OracleResultSet)stmt.executeQuery(SQLquery);
        rset.next();
        int nextval = rset.getInt("nextval");

        // reduce the number of round trip database communications
        String rowInsertSQL = "INSERT INTO image_table (id, image, signature) VALUES (" + nextval + 
                " , ordsys.ordimage.init(), ordsys.ordimagesignature.init())";
        stmt.execute(rowInsertSQL);

        // get the source ORDImage object (select first image)
        String srcSelectSQL = "SELECT image, signature FROM image_table WHERE id=(select min(id) from image_table)";
        rset = (OracleResultSet)stmt.executeQuery(srcSelectSQL);
        rset.next();
        OrdImage srcImageProxy = (OrdImage)rset.getCustomDatum("image", OrdImage.getFactory());
        // deprecated use: OrdImage srcImageProxy = (OrdImage)rset.getORAData("image", OrdImage.getORADataFactory());
        // OrdImageSignature srcSignatureProxy = (OrdImageSignature)rset.getCustomDatum("signature", OrdImage.getFactory());        
        rset.close();

        // get the newly inserted destination ORDImage object
        String dstSelectSQL = "SELECT image, signature FROM image_table where id=" + nextval + " FOR UPDATE";
        rset = (OracleResultSet)stmt.executeQuery(dstSelectSQL);
        rset.next();
        OrdImage dstImageProxy = (OrdImage)rset.getCustomDatum("image", OrdImage.getFactory());
        // deprecated use: OrdImage srcImageProxy = (OrdImage)rset.getORAData("image", OrdImage.getORADataFactory());
        OrdImageSignature dstSignatureProxy = (OrdImageSignature)rset.getCustomDatum("signature", OrdImageSignature.getFactory());        
        rset.close();
        
        // call the processCopy method (processing occurs on the SERVER)
        srcImageProxy.processCopy("maxscale=200 200 fileformat=jfif", dstImageProxy);
        
        // generate an image signature for the specified image
        dstSignatureProxy.generateSignature(dstImageProxy);
        
        // update the destination image in the second row
        String dstUpdateSQL = "UPDATE image_table SET image=?, signature=? WHERE id=" + nextval;
        OraclePreparedStatement opstmt = (OraclePreparedStatement)conn.prepareStatement(dstUpdateSQL);
        opstmt.setCustomDatum(1, dstImageProxy);
        opstmt.setCustomDatum(2, dstSignatureProxy);        
        opstmt.execute();
        opstmt.close();
                
        conn.commit();                 
/*
        // Test: select * from image_table; retrieving image properties
        SQLquery = "SELECT image FROM image_table WHERE id=" + nextval;
        rset = (OracleResultSet)stmt.executeQuery(SQLquery);
        rset.next();
        OrdImage imageProxy = (OrdImage)rset.getCustomDatum("image", OrdImage.getFactory());
        rset.close(); 

        int height = imageProxy.getHeight();
        int width  = imageProxy.getWidth();
        System.out.println("Image (width x height) = " + width + " x " + height);
*/
    }
     
    // download image from table into file  ------------------------------------
    public void imageDownload(String id, String fileName) throws SQLException, IOException {
        String SQLquery;
        OracleResultSet rset;
        
        // export the data in row 2
        String exportSelectSQL = "SELECT image FROM image_table WHERE id=" + id;

        rset = (OracleResultSet)stmt.executeQuery(exportSelectSQL);

        // get the proxy for the image in row 2
        rset.next();
        OrdImage imageProxy = (OrdImage)rset.getCustomDatum("image", OrdImage.getFactory());
        rset.close();

        // call the getDataInFile method to write the ORDImage in row 2 to disk
        imageProxy.getDataInFile(fileName);

        conn.commit(); 

    }

    // query the image content -------------------------------------------------
    public void imageQuery(String fileName) throws SQLException, IOException {
        /* similarity query...
         SELECT img1.id, img2.id, ORDSYS.IMGScore(123) as distance
           FROM image_table img1, image_table img2
           WHERE ORDSYS.IMGSimilar(img1.signature, img2.signature, 
             'color="0.3" texture="0.3" shape="0.3" location="0.1"', 100, 123) < 100
             AND img1.id <> img2.id
             AND img1.id = 8
           ORDER BY distance
        */

        // Note: use IMGSimilar and IMGScore rather than evaluateScore and isSimilar 
        /*
         CREATE INDEX image_index ON image_table(signature)
           INDEXTYPE IS ordsys.ordimageindex
           PARAMETERS ('ORDImage_Filter_Tablespace = USER1, ORDImage_Index_Tablespace = USER2')
         */ 

    }
     
    // main --------------------------------------------------------------------
    public static void main(String[] args) throws SQLException, IOException {
        Source2 source2 = new Source2();
        OracleResultSet rset;
        String SQLquery;
     
        // create a table with an key and ORDImage column to store images
        // source2.tableCreate();
        
        // upload an image from file into a table (1.jpg/2.gif)
        // source2.imageUpload("D:\\FIT\\Vyuka\\PDB\\Demo1\\Application1\\1.jpg");
        source2.imageUpload("D:\\FIT\\Vyuka\\PDB\\Demo1\\Application1\\2.gif");        
        
        // create thumbnail and change format
        // source2.imageConversion();
        
        // download image from table into file
        // source2.imageDownload("(select max(id) from image_table)", "D:\\FIT\\Vyuka\\PDB\\Demo1\\Application1\\thumb.jpg");
 
        // query the image content
        // source2.imageQuery("D:\\FIT\\Vyuka\\PDB\\Demo1\\Application1\\1.jpg");

    }
}
